@@ -14,117 +14,31 @@
  */
 
 #include <limits.h>
-#include <stdio.h>
-#include <stdint.h>
-#include <stddef.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
 #include <getopt.h>
 #include <inttypes.h>
+#include <stdbool.h>
 
-#include "rbtree.h"
-#include "list.h"
 #include "csum.h"
-#include "filerec.h"
-#include "hash-tree.h"
-#include "util.h"
 #include "dbfile.h"
-
-extern sqlite3 *gdb;
+#include "file_scan.h"
 
 unsigned int blocksize;
-int v2_hashfile = 0;
-static int version_only = 0;
-static int print_all_hashes = 0;
-static int print_blocks = 0;
+static bool print_all_hashes = false;
+static bool print_blocks = false;
+static bool show_block_hashes = false;
 static int num_to_print = 10;
-static int print_file_list = 0;
+static bool print_file_list = false;
 static char *serialize_fname = NULL;
-static struct dbfile_config dbfile_cfg;
 
-static sqlite3_stmt *top_hashes_stmt = NULL;
-static sqlite3_stmt *files_count_stmt = NULL;
-static sqlite3_stmt *find_blocks_stmt = NULL;
-
-/* dirty hack so we don't have to add file_scan.o to hashstats */
-int add_file_db(const char *filename [[maybe_unused]],
-		uint64_t inum [[maybe_unused]],
-		uint64_t subvolid [[maybe_unused]],
-		uint64_t size [[maybe_unused]],
-		uint64_t mtime [[maybe_unused]],
-		unsigned int seq [[maybe_unused]],
-		int *delete [[maybe_unused]])
-{
-	return 0;
-}
-
-static int prepare_statements(void)
-{
-	int ret;
-
-#define	FIND_TOP_HASHES							\
-"select digest, count(digest) from hashes group by digest having (count(digest) > 1) order by (count(digest)) desc;"
-	ret = sqlite3_prepare_v2(gdb, FIND_TOP_HASHES, -1, &top_hashes_stmt,
-				 NULL);
-	if (ret) {
-		fprintf(stderr, "error %d while prepping hash search stmt: %s\n",
-			ret, sqlite3_errstr(ret));
-		return ret;
-	}
-
-#define	FIND_FILES_COUNT						\
-"select count (distinct files.filename) from files INNER JOIN hashes on hashes.digest = ?1 AND files.subvol=hashes.subvol AND files.ino=hashes.ino;"
-	ret = sqlite3_prepare_v2(gdb, FIND_FILES_COUNT, -1, &files_count_stmt,
-				 NULL);
-	if (ret) {
-		fprintf(stderr, "error %d while preparing file count stmt: %s\n",
-			ret, sqlite3_errstr(ret));
-		return ret;
-	}
-
-#define	FIND_BLOCKS							\
-"select files.filename, hashes.loff, hashes.flags from files INNER JOIN hashes on hashes.digest = ?1 AND files.subvol=hashes.subvol AND files.ino=hashes.ino;"
-
-	ret = sqlite3_prepare_v2(gdb, FIND_BLOCKS, -1, &find_blocks_stmt, NULL);
-	if (ret) {
-		fprintf(stderr, "error %d while prepping find blocks stmt: %s\n",
-			ret, sqlite3_errstr(ret));
-		return ret;
-	}
-	return 0;
-}
-
-static void finalize_statements(void)
-{
-	sqlite3_finalize(top_hashes_stmt);
-	sqlite3_finalize(files_count_stmt);
-	sqlite3_finalize(find_blocks_stmt);
-}
-
-static void printf_file_block_flags(unsigned int flags)
-{
-	if (!flags)
-		return;
-
-	printf("( ");
-	if (flags & FILE_BLOCK_SKIP_COMPARE)
-		printf("skip_compare ");
-	if (flags & FILE_BLOCK_DEDUPED)
-		printf("deduped ");
-	if (flags & FILE_BLOCK_HOLE)
-		printf("hole ");
-	if (flags & FILE_BLOCK_PARTIAL)
-		printf("partial ");
-	printf(")");
-}
-
-static int print_all_blocks(unsigned char *digest)
+static int print_all_blocks(struct dbhandle *db, unsigned char *digest)
 {
 	int ret;
 	uint64_t loff;
-	unsigned int flags;
 	const unsigned char *filename;
+	_cleanup_(sqlite3_reset_stmt) sqlite3_stmt *find_blocks_stmt = db->stmts.find_blocks;
 
 	ret = sqlite3_bind_blob(find_blocks_stmt, 1, digest, DIGEST_LEN,
 				SQLITE_STATIC);
@@ -137,15 +51,10 @@ static int print_all_blocks(unsigned char *digest)
 	while ((ret = sqlite3_step(find_blocks_stmt)) == SQLITE_ROW) {
 		filename = sqlite3_column_text(find_blocks_stmt, 0);
 		loff = sqlite3_column_int64(find_blocks_stmt, 1);
-		flags = sqlite3_column_int(find_blocks_stmt, 2);
 
-		printf("  %s\tloff: %llu lblock: %llu "
-		       "flags: 0x%x ", filename,
+		printf("  %s\tloff: %llu lblock: %llu", filename,
 		       (unsigned long long)loff,
-		       (unsigned long long)loff / blocksize,
-		       flags);
-		printf_file_block_flags(flags);
-		printf("\n");
+		       (unsigned long long)loff / blocksize);
 	}
 	if (ret != SQLITE_DONE) {
 		fprintf(stderr,
@@ -154,18 +63,23 @@ static int print_all_blocks(unsigned char *digest)
 		return ret;
 	}
 
-	sqlite3_reset(find_blocks_stmt);
-
 	return 0;
 }
 
-static void print_by_size(void)
+static void print_by_size(struct dbhandle *db)
 {
 	int ret;
 	int header_printed = 0;
 	unsigned char *digest;
 	uint64_t count, files_count;
 
+	_cleanup_(sqlite3_reset_stmt) sqlite3_stmt *top_hashes_stmt;
+
+	if (show_block_hashes)
+		top_hashes_stmt = db->stmts.find_top_b_hashes;
+	else
+		top_hashes_stmt = db->stmts.find_top_e_hashes;
+
 	if (print_all_hashes)
 		printf("Print all hashes ");
 	else
@@ -177,22 +91,7 @@ static void print_by_size(void)
 		digest = (unsigned char *)sqlite3_column_blob(top_hashes_stmt, 0);
 		count = sqlite3_column_int64(top_hashes_stmt, 1);
 
-		ret = sqlite3_bind_blob(files_count_stmt, 1, digest, DIGEST_LEN,
-					SQLITE_STATIC);
-		if (ret) {
-			fprintf(stderr, "Error %d binding digest: %s\n", ret,
-				sqlite3_errstr(ret));
-			return;
-		}
-
-		ret = sqlite3_step(files_count_stmt);
-		if (ret != SQLITE_ROW && ret != SQLITE_DONE) {
-			fprintf(stderr, "error %d, file count search: %s\n",
-				ret, sqlite3_errstr(ret));
-			return;
-		}
-
-		files_count = sqlite3_column_int64(files_count_stmt, 0);
+		files_count = count_file_by_digest(db, digest, show_block_hashes);
 
 		if (!header_printed) {
 			printf("Hash, # Blocks, # Files\n");
@@ -202,10 +101,8 @@ static void print_by_size(void)
 		debug_print_digest(stdout, digest);
 		printf(", %"PRIu64", %"PRIu64"\n", count, files_count);
 
-		sqlite3_reset(files_count_stmt);
-
 		if (print_blocks) {
-			ret = print_all_blocks(digest);
+			ret = print_all_blocks(db, digest);
 			if (ret)
 				return;
 		}
@@ -231,40 +128,24 @@ static int print_files_cb(void *priv [[maybe_unused]], int argc,
 	return 0;
 }
 
-static void print_filerecs(void)
+static void print_file_info(char *fname, struct dbfile_config *cfg,
+				struct dbfile_stats *stats)
 {
-	int ret;
-	char *errorstr;
-
-#define	LIST_FILES							\
-"select ino, subvol, blocks, size, filename from files;"
-
-	printf("Showing %"PRIu64" files.\nInode\tSubvol ID\tBlocks Stored\tSize\tFilename\n",
-		dbfile_cfg.num_files);
-
-	ret = sqlite3_exec(gdb, LIST_FILES, print_files_cb, NULL, &errorstr);
-	if (ret) {
-		fprintf(stderr, "error %d, executing file search: %s\n", ret,
-			errorstr);
-		return;
-	}
+	printf("Raw header info for \"%s\":\n", fname);
+	printf("  version: %d.%d\tblock_size: %u\n", cfg->major,
+	       cfg->minor, cfg->blocksize);
+	printf("  num_files: %"PRIu64"\tnum_block_hashes: %"PRIu64"\tnum_extent_hashes: %"PRIu64"\n",
+	       stats->num_files, stats->num_b_hashes, stats->num_e_hashes);
 }
 
-static void print_file_info(void)
+static void version()
 {
-	printf("Raw header info for \"%s\":\n", serialize_fname);
-	printf("  version: %d.%d\tblock_size: %u\n", dbfile_cfg.major,
-	       dbfile_cfg.minor, dbfile_cfg.blocksize);
-	printf("  num_files: %"PRIu64"\tnum_hashes: %"PRIu64"\n",
-	       dbfile_cfg.num_files, dbfile_cfg.num_hashes);
+	printf("hashstats %s\n", VERSTRING);
 }
 
 static void usage(const char *prog)
 {
-	printf("hashstats %s\n", VERSTRING);
-	if (version_only)
-		return;
-
+	version();
 	printf("Print information about duperemove hashes.\n\n");
 	printf("Usage: %s [-n NUM] [-a] [-b] [-l] hashfile\n", prog);
 	printf("Where \"hashfile\" is a file generated by running duperemove\n");
@@ -274,6 +155,7 @@ static void usage(const char *prog)
 	printf("\t-n NUM\t\tPrint top N hashes, sorted by bucket size.\n");
 	printf("\t      \t\tDefault is 10.\n");
 	printf("\t-a\t\tPrint all hashes (overrides '-n', above)\n");
+	printf("\t-B\t\tShow block hashes, and not extents-hashes\n");
 	printf("\t-b\t\tPrint info on each block within our hash buckets\n");
 	printf("\t-l\t\tPrint a list of all files\n");
 	printf("\t--help\t\tPrints this help text.\n");
@@ -297,28 +179,30 @@ static int parse_options(int argc, char **argv)
 	if (argc < 2)
 		return 1;
 
-	while ((c = getopt_long(argc, argv, "labn:?", long_ops, NULL))
+	while ((c = getopt_long(argc, argv, "laBbn:?", long_ops, NULL))
 	       != -1) {
 		switch (c) {
 		case 'l':
-			print_file_list = 1;
+			print_file_list = true;
 			break;
 		case 'a':
-			print_all_hashes = 1;
+			print_all_hashes = true;
+			break;
+		case 'B':
+			show_block_hashes = true;
 			break;
 		case 'b':
-			print_blocks = 1;
+			print_blocks = true;
 			break;
 		case 'n':
 			num_to_print = atoi(optarg);
 			break;
 		case VERSION_OPTION:
-			version_only = 1;
-			return 1;
+			version();
+			exit(0);
 		case HELP_OPTION:
 		case '?':
 		default:
-			version_only = 0;
 			return 1;
 		}
 	}
@@ -334,37 +218,40 @@ static int parse_options(int argc, char **argv)
 int main(int argc, char **argv)
 {
 	int ret;
-	struct hash_tree tree;
-
-	init_filerec();
-	init_hash_tree(&tree);
+	struct dbfile_config dbfile_cfg;
+	struct dbfile_stats dbfile_stats;
 
 	if (parse_options(argc, argv)) {
 		usage(argv[0]);
 		return EINVAL;
 	}
 
-	ret = dbfile_open(serialize_fname, &dbfile_cfg);
+	_cleanup_(sqlite3_close_cleanup) struct dbhandle *db = dbfile_open_handle(serialize_fname);
+	if (!db) {
+		fprintf(stderr, "ERROR: Couldn't open db file %s\n",
+			serialize_fname);
+		return ENOMEM;
+	}
+
+	ret = dbfile_get_config(db->db, &dbfile_cfg);
+	if (ret)
+		return ret;
+
+	ret = dbfile_get_stats(db, &dbfile_stats);
 	if (ret)
 		return ret;
 
 	blocksize = dbfile_cfg.blocksize;
-
-	ret = prepare_statements();
-	if (ret)
-		return ret;
-
-	print_file_info();
+	print_file_info(serialize_fname, &dbfile_cfg, &dbfile_stats);
 
 	if (num_to_print || print_all_hashes)
-		print_by_size();
+		print_by_size(db);
 
-	if (print_file_list)
-		print_filerecs();
-
-	finalize_statements();
-
-	dbfile_close();
+	if (print_file_list) {
+		printf("Showing %"PRIu64" files.\nInode\tSubvol ID\tBlocks Stored\tSize\tFilename\n",
+			dbfile_stats.num_files);
+		dbfile_list_files(db, print_files_cb);
+	}
 
 	return ret;
 }
