@@ -38,29 +38,35 @@
 #include "util.h"
 #include "memstats.h"
 #include "debug.h"
+#include "dbfile.h"
+#include "fiemap.h"
 
 #include "run_dedupe.h"
 
-extern int dedupe_same_file;
-extern int fiemap_during_dedupe;
-
 static GMutex mutex;
 static GMutex console_mutex;
 static struct results_tree *results_tree;
 static volatile unsigned long long total_dedupe_passes;
 static volatile unsigned long long curr_dedupe_pass;
 static unsigned int leading_spaces;
+static bool whole_file_dedup;
 
-void print_dupes_table(struct results_tree *res)
+void print_dupes_table(struct results_tree *res, bool whole_file)
 {
 	struct rb_root *root = &res->root;
 	struct rb_node *node = rb_first(root);
 	struct dupe_extents *dext;
 	struct extent *extent;
+	char *kind;
+
+	if (whole_file)
+		kind = "files";
+	else
+		kind = "extents";
 
 	printf("Simple read and compare of file data found %u instances of "
-	       "extents that might benefit from deduplication.\n",
-	       res->num_dupes);
+	       "%s that might benefit from deduplication.\n",
+	       res->num_dupes, kind);
 
 	if (quiet || res->num_dupes == 0)
 		return;
@@ -71,8 +77,8 @@ void print_dupes_table(struct results_tree *res)
 
 		dext = rb_entry(node, struct dupe_extents, de_node);
 
-		printf("Showing %u identical extents of length %s with id ",
-		       dext->de_num_dupes, pretty_size(dext->de_len));
+		printf("Showing %u identical %s of length %s with id ",
+		       dext->de_num_dupes, kind, pretty_size(dext->de_len));
 		debug_print_digest_short(stdout, dext->de_hash);
 		printf("\n");
 		printf("Start\t\tFilename\n");
@@ -124,33 +130,6 @@ static void process_dedupe_results(struct dedupe_ctxt *ctxt,
 	}
 }
 
-#if 0
-static void get_extent_info(struct dupe_extents *dext)
-{
-       int ret = 0;
-       struct extent *extent;
-       struct filerec *file;
-
-       list_for_each_entry(extent, &dext->de_extents, e_list) {
-               file = extent->e_file;
-
-               if (filerec_open(file, 0))
-                       continue;
-
-               extent_shared_bytes(extent) = 0;
-               ret = filerec_count_shared(file, extent->e_loff, dext->de_len,
-                                          &extent_shared_bytes(extent),
-                                          &extent_poff(extent),
-                                          &extent_plen(extent));
-               if (ret) {
-                       fprintf(stderr, "%s: fiemap error %d: %s\n",
-                               extent->e_file->filename, ret, strerror(ret));
-               }
-               filerec_close(file);
-       }
-}
-#endif
-
 static void add_shared_extents(struct dupe_extents *dext, uint64_t *shared)
 {
 	struct extent *extent;
@@ -174,11 +153,11 @@ static void add_shared_extents_post(struct dupe_extents *dext, uint64_t *shared)
 
 	list_for_each_entry(extent, &dext->de_extents, e_list) {
 		file = extent->e_file;
-		ret = filerec_open(file, target_rw);
+		ret = filerec_open(file, true);
 		if (ret)
 			return;
 
-		ret = filerec_count_shared(file, extent->e_loff, dext->de_len,
+		ret = fiemap_count_shared(file->fd, extent->e_loff, extent->e_loff + dext->de_len,
 					   &bytes);
 
 		*shared += bytes;
@@ -325,27 +304,34 @@ static int dedupe_extent_list(struct dupe_extents *dext, uint64_t *fiemap_bytes,
 	 * goes below 2. If that happens, we return a special value so
 	 * the caller knows not to reference dext any more.
 	 */
-	if (fiemap_during_dedupe) {
-		clean_deduped(&dext);
-		if (!dext) {
-			qprintf("[%p] Skipping - extents are already deduped.\n",
-			       g_thread_self());
-			return DEDUPE_EXTENTS_CLEANED;
-		}
-
-		/*
-		 * Do this after clean_deduped as we may have removed some
-		 * extents.
-		 */
-		add_shared_extents(dext, &shared_prev);
+	clean_deduped(&dext);
+	if (!dext) {
+		qprintf("[%p] Skipping - extents are already deduped.\n",
+		       g_thread_self());
+		return DEDUPE_EXTENTS_CLEANED;
 	}
 
+	/*
+	 * Do this after clean_deduped as we may have removed some
+	 * extents.
+	 */
+	add_shared_extents(dext, &shared_prev);
+
 	list_for_each_entry(extent, &dext->de_extents, e_list) {
 		if (list_is_last(&extent->e_list, &dext->de_extents))
 			last = 1;
 
-		ret = filerec_open_once(extent->e_file, target_rw, &open_files);
+		ret = filerec_open_once(extent->e_file, &open_files);
 		if (ret) {
+			if (ret == ENOENT) {
+				/*
+				 * File were deleted. Maybe it was scanned
+				 * a long time ago. Let's clean the db.
+				 */
+				dbfile_lock();
+				dbfile_remove_file(dbfile_get_handle(), extent->e_file->filename);
+				dbfile_unlock();
+			}
 			fprintf(stderr, "%s: Skipping dedupe.\n",
 				extent->e_file->filename);
 			/*
@@ -412,7 +398,6 @@ static int dedupe_extent_list(struct dupe_extents *dext, uint64_t *fiemap_bytes,
 		}
 
 run_dedupe:
-
 		/*
 		 * We can get here with only the target extent (0
 		 * queued) for many reasons. Skip the dedupe in that
@@ -449,7 +434,7 @@ close_files:
 
 		if (!last) {
 			/* reopen target file as it got closed above */
-			ret = filerec_open_once(tgt_extent->e_file, target_rw,
+			ret = filerec_open_once(tgt_extent->e_file,
 						&open_files);
 			if (ret) {
 				fprintf(stderr,
@@ -463,8 +448,7 @@ close_files:
 	abort_on(ctxt != NULL);
 	abort_on(!RB_EMPTY_ROOT(&open_files.root));
 
-	if (fiemap_during_dedupe)
-		add_shared_extents_post(dext, &shared_post);
+	add_shared_extents_post(dext, &shared_post);
 
 	/*
 	 * It's entirely possible that some other process is
@@ -506,6 +490,9 @@ static int extent_dedupe_worker(struct dupe_extents *dext,
 	int ret;
 	unsigned long long passno = __atomic_add_fetch(&curr_dedupe_pass, 1, __ATOMIC_SEQ_CST);
 
+	struct extent *extent;
+	struct dbhandle *db = dbfile_get_handle();
+
 	ret = dedupe_extent_list(dext, fiemap_bytes, kern_bytes, passno);
 	if (ret) {
 		if (ret == DEDUPE_EXTENTS_CLEANED)
@@ -514,6 +501,27 @@ static int extent_dedupe_worker(struct dupe_extents *dext,
 		return ret;
 	}
 
+	dbfile_lock();
+	list_for_each_entry(extent, &dext->de_extents, e_list) {
+		if (whole_file_dedup) {
+			/* If we are deduping a whole file, then the extents may be remapped
+			 * by Linux. Let's drop them from the hashfile: even if some other file
+			 * share one on those extents, keeping the whole file deduplicated is
+			 * a better move.
+			 * TODO: do not delete the extents but rescan every files to fetch
+			 * the new extents mapping as well as their new hashes
+			 * This may cause dbfile_load_one_file_extent() to raise an error.
+			 */
+			dbfile_remove_extent_hashes(db, extent->e_file->inum, extent->e_file->subvolid);
+		} else {
+			/* Rescan physical offset and update the hashfile accordingly */
+			ret = fiemap_scan_extent(extent);
+			if (!ret)
+				dbfile_update_extent_poff(db, extent->e_file->inum, extent->e_file->subvolid, extent->e_loff, extent->e_poff);
+		}
+	}
+	dbfile_unlock();
+
 	if (!list_empty(&dext->de_extents)) {
 		g_mutex_lock(&mutex);
 		dupe_extents_free(dext, results_tree);
@@ -523,20 +531,17 @@ static int extent_dedupe_worker(struct dupe_extents *dext,
 	return 0;
 }
 
-static int dedupe_worker(void *priv, struct dedupe_counts *counts)
+static void dedupe_worker(void *priv, struct dedupe_counts *counts)
 {
-	int ret;
 	uint64_t fiemap_bytes = 0ULL;
 	uint64_t kern_bytes = 0ULL;
 
-	ret = extent_dedupe_worker(priv, &fiemap_bytes, &kern_bytes);
+	extent_dedupe_worker(priv, &fiemap_bytes, &kern_bytes);
 
 	g_mutex_lock(&dedupe_counts_mutex);
 	counts->fiemap_bytes += fiemap_bytes;
 	counts->kern_bytes += kern_bytes;
 	g_mutex_unlock(&dedupe_counts_mutex);
-
-	return ret;
 }
 
 static GThreadPool *dedupe_pool = NULL;
@@ -574,7 +579,7 @@ static int push_extents(struct results_tree *res)
 	return 0;
 }
 
-void dedupe_results(struct results_tree *res)
+void dedupe_results(struct results_tree *res, bool whole_file)
 {
 	int ret;
 	struct dedupe_counts counts = { 0ULL, };
@@ -587,17 +592,19 @@ void dedupe_results(struct results_tree *res)
 	curr_dedupe_pass = 0;
 	results_tree = res;
 
-	print_dupes_table(res);
+	whole_file_dedup = whole_file;
+
+	print_dupes_table(res, whole_file);
 
 	if (RB_EMPTY_ROOT(&res->root)) {
 		printf("Nothing to dedupe.\n");
 		return;
 	}
 
-	qprintf("Using %u threads for dedupe phase\n", io_threads);
+	qprintf("Using %u threads for dedupe phase\n", options.io_threads);
 
 	dedupe_pool = g_thread_pool_new((GFunc) dedupe_worker, &counts,
-					io_threads, TRUE, &err);
+					options.io_threads, TRUE, &err);
 	if (err) {
 		fprintf(stderr, "Unable to create dedupe thread pool: %s\n",
 			err->message);
@@ -619,10 +626,9 @@ void dedupe_results(struct results_tree *res)
 	if (ret == 0) {
 		vprintf("Kernel processed data (excludes target files): "
 			"%s\n", pretty_size(counts.kern_bytes));
-		if (fiemap_during_dedupe)
-			printf("Comparison of extent info shows a net "
-			       "change in shared extents of: %s\n",
-			       pretty_size(counts.fiemap_bytes));
+		printf("Comparison of extent info shows a net "
+		       "change in shared extents of: %s\n",
+		       pretty_size(counts.fiemap_bytes));
 	}
 }
 
@@ -635,7 +641,7 @@ int fdupes_dedupe(void)
 	OPEN_ONCE(open_files);
 
 	list_for_each_entry(file, &filerec_list, rec_list) {
-		ret = filerec_open_once(file, 0, &open_files);
+		ret = filerec_open_once(file, &open_files);
 		if (ret) {
 			fprintf(stderr, "%s: Skipping dedupe.\n",
 				file->filename);
