@@ -332,3 +332,10 @@ int increase_limits(void) {
 		(unsigned long long)new_r.rlim_cur);
 	return 0;
 }
+
+void debug_print_uuid(FILE *stream, uuid_t uuid)
+{
+	char buf[37];
+	uuid_unparse(uuid, buf);
+	fputs(buf, stream);
+}
