@@ -18,8 +18,12 @@
 #ifndef	__UTIL_H__
 #define	__UTIL_H__
 
+#include <stdlib.h>
+#include <dirent.h>
 #include <stdint.h>
 #include <sys/time.h>
+#include <unistd.h>
+#include <uuid/uuid.h>
 
 /* controlled by user options, turns pretty print on if true. */
 extern int human_readable;
@@ -56,4 +60,24 @@ void get_num_cpus(unsigned int *nr_phys, unsigned int *nr_log);
 /* Bump up maximum open file limit. */
 int increase_limits(void);
 
+#define _cleanup_(x) __attribute__((cleanup(x)))
+static inline void freep(void *p)
+{
+	free(*(void**) p);
+}
+
+static inline void closedirectory(DIR **p)
+{
+	if (*p)
+		closedir(*p);
+}
+
+static inline void closefd(int *fd)
+{
+	if (*fd >= 0)
+		close(*fd);
+}
+
+void debug_print_uuid(FILE *stream, uuid_t uuid);
+
 #endif	/* __UTIL_H__ */
